package com.xyh.mychatgpt.utils;

import com.unfbx.chatgpt.OpenAiClient;
import com.unfbx.chatgpt.OpenAiStreamClient;
import com.unfbx.chatgpt.entity.billing.CreditGrantsResponse;
import com.unfbx.chatgpt.entity.chat.ChatChoice;
import com.unfbx.chatgpt.entity.chat.ChatCompletion;
import com.unfbx.chatgpt.entity.chat.Message;
import com.unfbx.chatgpt.entity.common.Choice;
import com.unfbx.chatgpt.entity.completions.Completion;
import com.unfbx.chatgpt.interceptor.OpenAILogger;
import com.unfbx.chatgpt.interceptor.OpenAiResponseInterceptor;
import com.unfbx.chatgpt.sse.ConsoleEventSourceListener;
import okhttp3.OkHttpClient;
import okhttp3.logging.HttpLoggingInterceptor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.TimeUnit;

/**
 * @author xiangyuanhong
 * @description: TODO
 * @date 2023/3/21上午9:28
 */
@Component
public class ChatGPTUtils {

    @Value("${xyh.openai.key}")
    private  String token;
    @Autowired
    private RedisUtils redisUtils;

    public void ask(String model,String question,String uuid){
        StringBuffer result=new StringBuffer();
        try {
            HttpLoggingInterceptor httpLoggingInterceptor = new HttpLoggingInterceptor(new OpenAILogger());
            httpLoggingInterceptor.setLevel(HttpLoggingInterceptor.Level.BODY);
            OkHttpClient okHttpClient = new OkHttpClient
                    .Builder()
                    .addInterceptor(httpLoggingInterceptor)
                    .addInterceptor(new OpenAiResponseInterceptor())
                    .connectTimeout(5, TimeUnit.MINUTES)
                    .writeTimeout(5, TimeUnit.MINUTES)
                    .readTimeout(5, TimeUnit.MINUTES)
                    .build();
            OpenAiClient openAiClient = OpenAiClient.builder()
                    .apiKey(Arrays.asList(token))
                    .okHttpClient(okHttpClient)
                    .build();
            if(ChatCompletion.Model.GPT_3_5_TURBO.getName().equals(model)){
                // GPT-3.5-Turb模型
                Message message=Message.builder().role(Message.Role.USER).content(question).build();
                ChatCompletion chatCompletion = ChatCompletion.builder().messages(Arrays.asList(message)).build();
                List<ChatChoice> resultList = openAiClient.chatCompletion(chatCompletion).getChoices();
                for (int i = 0; i < resultList.size(); i++) {
                    result.append(resultList.get(i).getMessage().getContent());
                }
            }else{
                //text-davinci-003/text-ada-003/GPT-4/GPT-4-32K
                Completion completion = Completion.builder()
                        .prompt(question)
                        .model(model)
                        .maxTokens(2000)
                        .temperature(0)
                        .echo(false)
                        .build();
                Choice[] resultList = openAiClient.completions(completion).getChoices();
                for (Choice choice : resultList) {
                    result.append(choice.getText());
                }
            }
        }catch (Exception e) {
            System.out.println(e.getMessage());
            result.append("小爱还不太懂，回去一定努力学习补充知识");
        }
        redisUtils.set(uuid,result.toString());
    }

    public void askForStream(){
        try {
            HttpLoggingInterceptor httpLoggingInterceptor = new HttpLoggingInterceptor(new OpenAILogger());
            httpLoggingInterceptor.setLevel(HttpLoggingInterceptor.Level.BODY);
            OkHttpClient okHttpClient = new OkHttpClient
                    .Builder()
                    .addInterceptor(httpLoggingInterceptor)
                    .addInterceptor(new OpenAiResponseInterceptor())
                    .connectTimeout(5, TimeUnit.MINUTES)
                    .writeTimeout(5, TimeUnit.MINUTES)
                    .readTimeout(5, TimeUnit.MINUTES)
                    .build();
            ConsoleEventSourceListener eventSourceListener = new ConsoleEventSourceListener();
            OpenAiStreamClient openAiStreamClient= OpenAiStreamClient.builder()
                    .apiKey(Arrays.asList(token))
                    .okHttpClient(okHttpClient)
                    .build();
            Message message=Message.builder().role(Message.Role.USER).content("今天是几号").build();
            ChatCompletion chatCompletion = ChatCompletion.builder().messages(Arrays.asList(message)).build();
            openAiStreamClient.streamChatCompletion(chatCompletion, eventSourceListener);
            CountDownLatch countDownLatch = new CountDownLatch(1);
            countDownLatch.await();
        }catch (InterruptedException e) {
            System.out.println(e.getMessage());
        }
    }


    /**
     * 查询账号金额
     * @return
     */
    public Map<String,Object> creditGrants(){
        Map<String,Object> map=new HashMap<>();
        HttpLoggingInterceptor httpLoggingInterceptor = new HttpLoggingInterceptor(new OpenAILogger());
        httpLoggingInterceptor.setLevel(HttpLoggingInterceptor.Level.BODY);
        OkHttpClient okHttpClient = new OkHttpClient
                .Builder()
                .addInterceptor(httpLoggingInterceptor)
                .addInterceptor(new OpenAiResponseInterceptor())
                .connectTimeout(5, TimeUnit.MINUTES)
                .writeTimeout(5, TimeUnit.MINUTES)
                .readTimeout(5, TimeUnit.MINUTES)
                .build();
        OpenAiClient openAiClient = OpenAiClient.builder()
                .apiKey(Arrays.asList(token))
                .okHttpClient(okHttpClient)
                .build();
        CreditGrantsResponse creditGrantsResponse= openAiClient.creditGrants();
        map.put("totalGranted",creditGrantsResponse.getTotalGranted());
        map.put("totalUsed",creditGrantsResponse.getTotalUsed());
        map.put("totalAvailable",creditGrantsResponse.getTotalAvailable());
        return map;
    }


}
